#University of California San Francisco
#Supplemental for
#
#A model for specific and nonspecific binding of ligand to multi-protein complexes by native mass spectrometry
#
#Shenheng Guan, et al
#2015
#
#command line:...>deconvKas_general_curveFit.py Kaps_result.txt 1
#
import sys
import math
import numpy as np
import copy
import math
from scipy.optimize import fsolve,fmin
import matplotlib.pyplot as plt

##mode =0
##fn='Kaps_result.txt'
fn=sys.argv[1]
mode=int(sys.argv[2])#mode: 0=constant; 1=inverse power

def readData(fn):
    fid=open(fn,'r')
    data=[]
    CalConc=fid.readline().strip('\n').split('\t')[1]
    data.append(CalConc)
    NoOfLigands=[int(float(x)) for x in fid.readline().strip('\n').split('\t')[2:]]
    data.append(NoOfLigands)
    Kaps=[float(x) for x in fid.readline().strip('\n').split('\t')[2:]]
    data.append(Kaps)
    return data
    
def powerLaw(x,p0,p1,p2):
    y=np.zeros(x.size)
    for j in range(0,y.size):
        y[j]=p0/(float(j+1))**p1
    y[0]+=p2
    return y

def constLaw(x,p0,p2):
    y=np.zeros(x.size)
    for j in range(0,y.size):
        y[j]=p0
    y[0]+=p2
    return y

def leastSqError(expt,pred):#Pearson's chi-squared
    return np.dot((expt-pred)/pred,(expt-pred)/pred)

#get data
data=readData(fn)
x=np.array(data[1])
y=np.array(data[-1])

#curve fitting
from scipy.optimize import curve_fit
if mode==1:
    w, covH, infodict, errmsg, ier = curve_fit(powerLaw,x,y,full_output=True)
    z0=powerLaw(x,w[0],w[1],w[2])
else:
    w, covH, infodict, errmsg, ier = curve_fit(constLaw,x,y,full_output=True)
    z0=constLaw(x,w[0],w[1])

z=copy.deepcopy(z0)

#compute stats
R2=1.-sum((y-z)*(y-z))/sum((y-sum(y))*(y-sum(y)))
R2adj=1.-(len(y)-1.)/(len(y)-len(w))*(1.-R2)

from scipy import stats
DF=len(y)-len(w)
t_factor=stats.t.ppf(0.95, DF)#t-distribution (or DF) adjustment

SE=[]#standard error (actual is sqrt of covH[j,j])
SS=leastSqError(y,z)
for j in range(0,len(w)):
    SE.append(np.sqrt(SS/DF*np.abs(covH[j,j])))

dw=[]#95% confidence interval
for j in range(0,len(w)):
    dw.append(SE[j]*t_factor)
    
print '\n'
if mode == 0:
    print 'Constant non-specific binding model'
else:
    print 'inverse power non-specific binding model'
print 'Ks= %4.2g +- %4.2g'%(w[-1],dw[-1])
print 'beta= %4.2g +- %4.2g'%(w[0],dw[0])
if mode==1:
    print 'gamma= %4.2f +- %4.2f'%(w[1],dw[1])
print 'R2= %8.5f, R2adj= %8.5f'%(R2,R2adj)
print 'AIC= %8.5f, AICc= %8.5f'%(AIC,AICc)

#plot data
z0[0]-=w[-1]
z1=np.zeros(x.size)
z1[0]=w[-1]
width=0.35
plt.bar(x,z0,width,color='g')
plt.bar(x,z1,width,color='r',bottom=z0)
x1=x+width/2.
plt.plot(x1,y,'o',color='c',marker='o',markersize=10)
plt.xticks(x1,x.astype(int),fontsize=14)
plt.ylim(0,y[0]*1.1)
plt.xlabel('# of Ligands',fontsize=20)
plt.ylabel('Associate Constant (1/M)',fontsize=20)
plt.show()






